from typing import Type, List

import os
import torch


class ModelDeploymentConfig:
    def __init__(
            self, model_class: Type[torch.nn.Module], path: str, port: int, allocations: List[int],
            use_torchscript: bool = True
    ):
        self.model_class = model_class
        self.port = port
        self.path = os.path.join('mnt', 'components', path)
        self.allocations = allocations
        self.use_torchscript = use_torchscript



from typing import Dict

import subprocess

from .multiprocess_api import ModelDeploymentConfig


def run_model_fleet(model_deployments: Dict[str, ModelDeploymentConfig]):
    for model_config in model_deployments.values():
        model_class = model_config.model_class
        for worker in range(len(model_config.allocations)):
            subprocess.Popen([
                'python', '-m', 'fastformers.run_server',
                '--model', "{0}.{1}".format(model_class.__module__, model_class.__name__),
                '--path', model_config.path,
                '--device', str(model_config.allocations[worker]),
                '--port', str(model_config.port + worker),
                f'--{"" if model_config.use_torchscript else "no-"}torchscript']
            )



from typing import Type

import argparse
import torch
import importlib

from .multiprocess_api import ModelServer


def init_parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser()
    parser.add_argument('--model', type=str, required=True)
    parser.add_argument('--path', type=str, required=True)
    parser.add_argument('--port', type=int, required=True)
    parser.add_argument('--device', type=int, required=True)
    parser.add_argument('--torchscript', dest='torchscript', action='store_true')
    parser.add_argument('--no-torchscript', dest='torchscript', action='store_false')
    return parser


def run_server():
    args = init_parser().parse_args()
    model_name: str = args.model
    split_name = model_name.split('.')
    module_name = '.'.join(split_name[:-1])
    class_name = split_name[-1]
    port: int = args.port
    device = f'cuda:{args.device}'
    model_class: Type[torch.nn.Module] = getattr(importlib.import_module(module_name), class_name)
    model = model_class.from_pretrained(args.path).to(device)
    if args.torchscript:
        model = torch.jit.script(model)
    server = ModelServer(port=port, model=model, model_name=class_name)
    try:
        server.run()
    finally:
        server.close()


if __name__ == '__main__':
    run_server()
